@@ -7,6 +7,7 @@
 #include "pdf.h"
 #include "rng.h"
 #include "mathinline.h"
+#include <utility>
 
 struct hit_record;
 
@@ -14,22 +15,17 @@ vec3 reflect(const vec3& v, const vec3& n) {
   return(v - 2*dot(v,n) * n);
 }
 
-Float schlick(Float cosine, Float ref_idx) {
-  Float r0 = (1 - ref_idx) / (1 + ref_idx);
+Float schlick(Float cosine, Float ref_idx, Float ref_idx2) {
+  Float r0 = (ref_idx2 - ref_idx) / (ref_idx2 + ref_idx);
   r0 = r0 * r0;
   return(r0 + (1-r0) * pow((1-cosine),5));
 }
 
-bool refract(const vec3& v, const vec3& n, Float ni_over_nt, vec3& refracted) {
-  vec3 uv = unit_vector(v);
-  Float dt = dot(uv, n);
-  Float discriminant = 1.0 - ni_over_nt * ni_over_nt * (1 - dt * dt);
-  if(discriminant > 0) {
-    refracted = ni_over_nt * (uv - n * dt) - n * sqrt(discriminant);
-    return(true);
-  } else {
-    return(false);
-  }
+vec3 refract(const vec3& uv, const vec3& n, Float ni_over_nt) {
+  Float cos_theta = dot(-uv, n);
+  vec3 r_out_parallel =  ni_over_nt * (uv + cos_theta*n);
+  vec3 r_out_perp = -sqrt(1.0 - r_out_parallel.squared_length()) * n;
+  return(r_out_parallel + r_out_perp);
 }
 
 struct scatter_record {
@@ -54,9 +50,16 @@ class material {
     virtual ~material() {};
 };
 
+
 class lambertian : public material {
   public: 
-    lambertian(texture *a) : albedo(a) {}
+    // lambertian(texture *a) : albedo(a), hasAlphaTexture(false), alphaTexture(nullptr) {
+    //   Rcpp::Rcout << "inside no alpha " << hasAlphaTexture << " " << typeid(alphaTexture).name() << "\n";
+    // 
+    // }
+    lambertian(texture *a) : albedo(a) {
+      // Rcpp::Rcout << "inside alpha " << hasAlphaTexture << " " << typeid(alphaTexture).name() << "\n";
+    }
     ~lambertian() {
       // Rcpp::Rcout << "delete albedo" << "\n";
       if(albedo) delete albedo;
@@ -82,10 +85,12 @@ class lambertian : public material {
 
 class metal : public material {
   public:
-    metal(const vec3& a, Float f) : albedo(a) { if (f < 1) fuzz = f; else fuzz = 1;}
+    metal(const vec3& a, Float f) : albedo(a) { 
+      if (f < 1) fuzz = f; else fuzz = 1;
+    }
     virtual bool scatter(const ray& r_in, const hit_record& hrec, scatter_record& srec, random_gen& rng) {
       vec3 reflected = reflect(unit_vector(r_in.direction()),hrec.normal);
-      srec.specular_ray = ray(hrec.p, reflected + fuzz * rng.random_in_unit_sphere(), r_in.time());
+      srec.specular_ray = ray(hrec.p, reflected + fuzz * rng.random_in_unit_sphere(), r_in.pri_stack, r_in.time());
       srec.attenuation = albedo;
       srec.is_specular = true;
       srec.pdf_ptr = 0;
@@ -97,39 +102,107 @@ class metal : public material {
 // 
 class dielectric : public material {
   public:
-    dielectric(const vec3& a, Float ri, random_gen& rng) : ref_idx(ri), albedo(a), rng(rng) {};
+    dielectric(const vec3& a, Float ri, const vec3& atten, int priority2, random_gen& rng) : ref_idx(ri), 
+               albedo(a), attenuation(atten), priority(priority2), rng(rng) {};
     virtual bool scatter(const ray& r_in, const hit_record& hrec, scatter_record& srec, random_gen& rng) {
       srec.is_specular = true;
       vec3 outward_normal;
       vec3 reflected = reflect(r_in.direction(), hrec.normal);
       Float ni_over_nt;
       srec.attenuation = albedo;
-      vec3 refracted;
+      Float current_ref_idx = 1.0;
+      
+      size_t active_priority_value = priority;
+      size_t next_down_priority = 100000;
+      int current_layer = -1; //keeping track of index of current material
+      int prev_active = -1;  //keeping track of index of active material (higher priority)
+
+      bool entering = dot(hrec.normal, r_in.direction()) < 0;
+      bool skip = false;
+      
+      for(size_t i = 0; i < r_in.pri_stack->size(); i++) {
+        if(r_in.pri_stack->at(i) == this) {
+          current_layer = i;
+          continue;
+        }
+        if(r_in.pri_stack->at(i)->priority < active_priority_value) {
+          active_priority_value = r_in.pri_stack->at(i)->priority;
+          skip = true;
+        }
+        if(r_in.pri_stack->at(i)->priority < next_down_priority && r_in.pri_stack->at(i) != this) {
+          prev_active = i;
+          next_down_priority = r_in.pri_stack->at(i)->priority;
+        }
+      }
+      if(entering) {
+        r_in.pri_stack->push_back(this);
+      }
+      current_ref_idx = prev_active != -1 ? r_in.pri_stack->at(prev_active)->ref_idx : 1;
+      if(skip) {
+        srec.specular_ray = ray(hrec.p, r_in.direction(), r_in.pri_stack, r_in.time());
+        Float distance = (hrec.p-r_in.point_at_parameter(0)).length();
+        vec3 prev_atten = r_in.pri_stack->at(prev_active)->attenuation;
+        srec.attenuation = vec3(std::exp(-distance * prev_atten.x()),
+                                std::exp(-distance * prev_atten.y()),
+                                std::exp(-distance * prev_atten.z()));
+        if(!entering && current_layer != -1) {
+          r_in.pri_stack->erase(r_in.pri_stack->begin() + static_cast<size_t>(current_layer));
+        }
+        return(true);
+      }
+      
       Float reflect_prob;
-      Float cosine;
-      if(dot(r_in.direction(), hrec.normal) > 0) {
+      if(!entering) {
         outward_normal = -hrec.normal;
-        ni_over_nt = ref_idx;
-        cosine = ref_idx * dot(r_in.direction(), hrec.normal) / r_in.direction().length();
+        ni_over_nt = ref_idx / current_ref_idx ;
       } else {
         outward_normal = hrec.normal;
-        ni_over_nt = 1.0 / ref_idx;
-        cosine = -dot(r_in.direction(), hrec.normal) / r_in.direction().length();
+        ni_over_nt = current_ref_idx / ref_idx;
       }
-      if(refract(r_in.direction(), outward_normal, ni_over_nt, refracted)) {
-        reflect_prob = schlick(cosine, ref_idx);
+      vec3 unit_direction = unit_vector(r_in.direction());
+      Float cos_theta = ffmin(dot(-unit_direction, outward_normal), (Float)1.0);
+      Float sin_theta = sqrt(1.0 - cos_theta*cos_theta);
+      if(ni_over_nt * sin_theta <= 1.0 ) {
+        reflect_prob = schlick(cos_theta, ref_idx, current_ref_idx);
+        reflect_prob = ni_over_nt == 1 ? 0 : reflect_prob;
       } else {
         reflect_prob = 1.0;
       }
+      if(!entering) {
+        Float distance = (hrec.p-r_in.point_at_parameter(0)).length();
+        srec.attenuation = vec3(std::exp(-distance * attenuation.x()),
+                                std::exp(-distance * attenuation.y()),
+                                std::exp(-distance * attenuation.z()));
+      } else {
+        if(prev_active != -1) {
+          Float distance = (hrec.p-r_in.point_at_parameter(0)).length();
+          vec3 prev_atten = r_in.pri_stack->at(prev_active)->attenuation;
+
+          srec.attenuation = albedo * vec3(std::exp(-distance * prev_atten.x()),
+                                  std::exp(-distance * prev_atten.y()),
+                                  std::exp(-distance * prev_atten.z()));
+        } else {
+          srec.attenuation = albedo;
+        }
+      }
       if(rng.unif_rand() < reflect_prob) {
-        srec.specular_ray = ray(hrec.p, reflected, r_in.time());
-      }  else {
-        srec.specular_ray = ray(hrec.p, refracted, r_in.time());
+        if(entering) {
+          r_in.pri_stack->pop_back();
+        }
+        srec.specular_ray = ray(hrec.p, reflected, r_in.pri_stack, r_in.time());
+      } else {
+        if(!entering && current_layer != -1) {
+          r_in.pri_stack->erase(r_in.pri_stack->begin() + current_layer);
+        }
+        vec3 refracted = refract(unit_direction, outward_normal, ni_over_nt);
+        srec.specular_ray = ray(hrec.p, refracted, r_in.pri_stack, r_in.time());
       }
       return(true);
     }
     Float ref_idx;
     vec3 albedo;
+    vec3 attenuation;
+    size_t priority;
     random_gen rng;
 };
 
@@ -162,7 +235,7 @@ public:
   }
   virtual bool scatter(const ray& r_in, const hit_record& rec, scatter_record& srec, random_gen& rng) {
     srec.is_specular = true;
-    srec.specular_ray = ray(rec.p, rng.random_in_unit_sphere());
+    srec.specular_ray = ray(rec.p, rng.random_in_unit_sphere(), r_in.pri_stack);
     srec.attenuation = albedo->value(rec.u,rec.v,rec.p);
     return(true);
   }
@@ -174,7 +247,7 @@ public:
 
 class orennayar : public material {
 public:
-  orennayar(texture *a, Float sigma) : albedo(a) {
+  orennayar(texture *a, Float sigma) : albedo(a)  {
     Float sigma2 = sigma*sigma;
     A = 1.0f - (sigma2 / (2.0f * (sigma2 + 0.33f)));
     B = 0.45f * sigma2 / (sigma2 + 0.09f);
