@@ -11,8 +11,12 @@ class sphere: public hitable {
     ~sphere() {
       // Rcpp::Rcout << "sphere delete " << typeid(*mat_ptr).name() << "\n";
       delete mat_ptr;
+      delete alpha_mask;
     }
-    sphere(vec3 cen, Float r, material *mat) : center(cen), radius(r), mat_ptr(mat) {};
+    sphere(vec3 cen, Float r, material *mat, alpha_texture *alpha_mask) : center(cen), radius(r), 
+           mat_ptr(mat), alpha_mask(alpha_mask) {
+      // Rcpp::Rcout << "sphere create " <<  typeid(*mat_ptr).name() << " " << mat_ptr->hasAlphaTexture << "\n";
+    };
     virtual bool hit(const ray& r, Float tmin, Float tmax, hit_record& rec, random_gen& rng);
     virtual bool bounding_box(Float t0, Float t1, aabb& box) const;
     virtual Float pdf_value(const vec3& o, const vec3& v, random_gen& rng);
@@ -20,6 +24,7 @@ class sphere: public hitable {
     vec3 center;
     Float radius;
     material *mat_ptr;
+    alpha_texture *alpha_mask;
 };
 
 bool sphere::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, random_gen& rng) {
@@ -31,7 +36,34 @@ bool sphere::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, random
   if (!quadratic(a, b, c, &temp1, &temp2)) {
     return(false);
   }
-  if(temp1 < t_max && temp1 > t_min) {
+  bool is_hit = true;
+  bool second_is_hit = true;
+  if(alpha_mask) {
+    Float u;
+    Float v;
+    if(temp1 < t_max && temp1 > t_min) {
+      vec3 p1 = r.point_at_parameter(temp1);
+      p1 *= radius / p1.length(); 
+      vec3 normal = (p1 - center) / radius;
+      get_sphere_uv(normal, u, v);
+      if(alpha_mask->value(u, v, rec.p).x() < rng.unif_rand()) {
+        is_hit = false;
+      }
+    }
+    if(temp2 < t_max && temp2 > t_min) {
+      vec3 p2 = r.point_at_parameter(temp2);
+      p2 *= radius / p2.length(); 
+      vec3 normal = (p2 - center) / radius;
+      get_sphere_uv(normal, u, v);
+      if(alpha_mask->value(u, v, rec.p).x() < rng.unif_rand()) {
+        if(!is_hit) {
+          return(false);
+        }
+        second_is_hit = false;
+      } 
+    }
+  }
+  if(temp1 < t_max && temp1 > t_min && is_hit) {
     rec.t = temp1;
     rec.p = r.point_at_parameter(rec.t);
     rec.p *= radius / rec.p.length(); 
@@ -40,12 +72,15 @@ bool sphere::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, random
     rec.mat_ptr = mat_ptr;
     return(true);
   }
-  if(temp2 < t_max && temp2 > t_min) {
+  if(temp2 < t_max && temp2 > t_min && second_is_hit) {
     rec.t = temp2;
     rec.p = r.point_at_parameter(rec.t);
-    rec.p *= radius / rec.p.length(); 
+    rec.p *= radius / rec.p.length();
     rec.normal = (rec.p - center) / radius;
     get_sphere_uv(rec.normal, rec.u, rec.v);
+    if(alpha_mask) {
+      rec.normal = -rec.normal;
+    }
     rec.mat_ptr = mat_ptr;
     return(true);
   }
@@ -79,8 +114,10 @@ bool sphere::bounding_box(Float t0, Float t1, aabb& box) const {
 class moving_sphere: public hitable {
   public:
     moving_sphere() {}
-    moving_sphere(vec3 cen0, vec3 cen1, Float t0, Float t1, Float r, material *mat) : 
-    center0(cen0), center1(cen1), time0(t0), time1(t1), radius(r), mat_ptr(mat) {};
+    moving_sphere(vec3 cen0, vec3 cen1, Float t0, Float t1, Float r, 
+                  material *mat,alpha_texture *alpha_mask) : 
+                  center0(cen0), center1(cen1), time0(t0), time1(t1), radius(r), 
+                  mat_ptr(mat), alpha_mask(alpha_mask) {};
     virtual bool hit(const ray& r, Float tmin, Float tmax, hit_record& rec, random_gen& rng);
     virtual bool bounding_box(Float t0, Float t1, aabb& box) const;
     vec3 center(Float time) const;
@@ -88,6 +125,7 @@ class moving_sphere: public hitable {
     Float time0, time1;
     Float radius;
     material *mat_ptr;
+    alpha_texture *alpha_mask;
 };
 
 vec3 moving_sphere::center(Float time) const {
@@ -110,7 +148,32 @@ bool moving_sphere::hit(const ray& r, Float t_min, Float t_max, hit_record& rec,
   if (!quadratic(a, b, c, &temp1, &temp2)) {
     return(false);
   }
-  if(temp1 < t_max && temp1 > t_min) {
+  bool is_hit = true;
+  if(alpha_mask) {
+    Float u;
+    Float v;
+    if(temp1 < t_max && temp1 > t_min) {
+      vec3 p1 = r.point_at_parameter(temp1);
+      p1 *= radius / p1.length(); 
+      vec3 normal = (p1 - center(r.time())) / radius;
+      get_sphere_uv(normal, u, v);
+      if(alpha_mask->value(u, v, rec.p).x() < rng.unif_rand()) {
+        is_hit = false;
+      }
+    }
+    if(temp2 < t_max && temp2 > t_min) {
+      vec3 p2 = r.point_at_parameter(temp2);
+      p2 *= radius / p2.length(); 
+      vec3 normal = (p2 - center(r.time())) / radius;
+      get_sphere_uv(normal, u, v);
+      if(alpha_mask->value(u, v, rec.p).x() < rng.unif_rand()) {
+        if(!is_hit) {
+          return(false);
+        }
+      } 
+    }
+  }
+  if(temp1 < t_max && temp1 > t_min && is_hit) {
     rec.t = temp1;
     rec.p = r.point_at_parameter(rec.t);
     rec.p *= radius / rec.p.length(); 
@@ -125,6 +188,9 @@ bool moving_sphere::hit(const ray& r, Float t_min, Float t_max, hit_record& rec,
     rec.p *= radius / rec.p.length(); 
     rec.normal = (rec.p - center(r.time())) / radius;
     get_sphere_uv(rec.normal, rec.u, rec.v);
+    if(!is_hit) {
+      rec.normal = -rec.normal;
+    }
     rec.mat_ptr = mat_ptr;
     return(true);
   }
