@@ -7,8 +7,6 @@
 #include "tinyobj/tiny_obj_loader.h"
 #include "stb_image.h"
 #include <Rcpp.h>
-#include <chrono>
-#include <thread>
 
 inline char separator() {
   #if defined _WIN32 || defined __CYGWIN__
@@ -22,14 +20,11 @@ class trimesh : public hitable {
 public:
   trimesh() {}
   ~trimesh() {
-    // Rcpp::Rcout << "deleting trimesh " << obj_materials.size() << "\n";
     delete tri_mesh_bvh;
     for(auto mat : obj_materials) {
       if(mat) stbi_image_free(mat);
     }
-    if(mat_ptr) {
-      delete mat_ptr;
-    }
+    delete mat_ptr;
   }
   trimesh(std::string inputfile, std::string basedir, Float scale, 
           Float shutteropen, Float shutterclose, random_gen rng) {
@@ -47,6 +42,7 @@ public:
       for (size_t s = 0; s < shapes.size(); s++) {
         n += shapes[s].mesh.num_face_vertices.size();
       }
+      alpha_texture* alpha = nullptr;
       obj_materials.reserve(materials.size()+1);
       // std::vector<Float* > obj_materials(materials.size()+1);
       std::vector<vec3 > diffuse_materials(materials.size()+1);
@@ -55,12 +51,13 @@ public:
       std::vector<bool > has_diffuse(materials.size()+1);
       std::vector<bool > has_transparency(materials.size()+1);
       std::vector<bool > has_single_diffuse(materials.size()+1);
+      std::vector<bool > has_alpha(materials.size()+1);
       std::vector<vec3 > image_dim(materials.size()+1);
       std::vector<int > nx_mat(materials.size()+1);
       std::vector<int > ny_mat(materials.size()+1);
       std::vector<int > nn_mat(materials.size()+1);
       
-      int nx,ny,nn;
+      int nx, ny, nn;
 
       for (size_t i = 0; i < materials.size(); i++) {
         if(strlen(materials[i].diffuse_texname.c_str()) > 0) {
@@ -70,27 +67,49 @@ public:
           nx_mat[i] = nx;
           ny_mat[i] = ny;
           nn_mat[i] = nn;
+          has_alpha[i] = false;
+          if(nn == 4) {
+            for(int j = 0; j < nx - 1; j++) {
+              for(int k = 0; k < ny - 1; k++) {
+                if(obj_materials[i][4*j + 4*nx*k + 3] != 1.0) {
+                  has_alpha[i] = true;
+                  break;
+                }
+              }
+              if(has_alpha[i]) {
+                break;
+              }
+            }
+          } 
         } else if (sizeof(materials[i].diffuse) == 12) {
           obj_materials.push_back(nullptr);
+          alpha_materials.push_back(nullptr);
+          
           diffuse_materials[i] = vec3(materials[i].diffuse[0],materials[i].diffuse[1],materials[i].diffuse[2]);
           has_diffuse[i] = true;
+          has_alpha[i] = false;
           has_single_diffuse[i] = true;
         } else {
           obj_materials.push_back(nullptr);
+          alpha_materials.push_back(nullptr);
+          
           has_diffuse[i] = false;
+          has_alpha[i] = false;
           has_single_diffuse[i] = false;
         }
         if(materials[i].dissolve < 1) {
           obj_materials.push_back(nullptr);
+          alpha_materials.push_back(nullptr);
+          
           specular_materials[i] = vec3(materials[i].diffuse[0],materials[i].diffuse[1],materials[i].diffuse[2]);
           ior_materials[i] = materials[i].ior;
+          has_alpha[i] = false;
           has_transparency[i] = true; 
         }
       }
       bool has_normals = attrib.normals.size() > 0 ? true : false;
       vec3 tris[3];
       vec3 normals[3];
-      // vec3 colors[3];
       Float tx[3];
       Float ty[3];
       triangles.reserve(n+1);
@@ -106,6 +125,7 @@ public:
             tris[v] = vec3(attrib.vertices[3*idx.vertex_index+0],
                            attrib.vertices[3*idx.vertex_index+1],
                            attrib.vertices[3*idx.vertex_index+2])*scale;
+
             if(has_normals && idx.normal_index != -1) {
               tempnormal = true;
               normals[v] = vec3(attrib.normals[3*idx.normal_index+0],
@@ -115,13 +135,12 @@ public:
             if(idx.texcoord_index != -1) {
               tx[v] = attrib.texcoords[2*idx.texcoord_index+0];
               ty[v] = attrib.texcoords[2*idx.texcoord_index+1];
-              if(tx[v] < 0) tx[v] += 1;
-              if(ty[v] < 0) ty[v] += 1;
-              if(tx[v] > 1) tx[v] -= 1;
-              if(ty[v] > 1) ty[v] -= 1;
+              while(tx[v] < 0) tx[v] += 1;
+              while(ty[v] < 0) ty[v] += 1;
+              while(tx[v] > 1) tx[v] -= 1;
+              while(ty[v] > 1) ty[v] -= 1;
             }
           }
-          
           index_offset += 3;
           if(std::isnan(tris[0].x()) || std::isnan(tris[0].y()) || std::isnan(tris[0].z()) ||
              std::isnan(tris[1].x()) || std::isnan(tris[1].y()) || std::isnan(tris[1].z()) ||
@@ -132,6 +151,18 @@ public:
           }
           // per-face material
           int material_num = shapes[s].mesh.material_ids[f];
+          if(material_num > -1) {
+            if(has_alpha[material_num]) {
+              alpha = new alpha_texture(obj_materials[material_num], nx, ny, nn, 
+                                        vec3(tx[0], tx[1], tx[2]), 
+                                        vec3(ty[0], ty[1], ty[2]));
+              alpha_materials.push_back(alpha);
+            } else {
+              alpha = nullptr;
+            }
+          } else {
+            alpha = nullptr;
+          }
           
           material *tex;
           
@@ -140,7 +171,8 @@ public:
               tex = new lambertian(new constant_texture(vec3(1,1,1)));
             } else {
               if(has_transparency[material_num]) {
-                tex = new dielectric(specular_materials[material_num], ior_materials[material_num], rng);;
+                tex = new dielectric(specular_materials[material_num], ior_materials[material_num], vec3(0,0,0), 
+                                     0,  rng);;
               } else if(has_diffuse[material_num]) {
                 if(has_single_diffuse[material_num]) {
                   tex = new lambertian(new constant_texture(diffuse_materials[material_num]));
@@ -153,13 +185,15 @@ public:
                 tex = new lambertian(new constant_texture(vec3(1,1,1)));
               }
             }
-            triangles.push_back(new triangle(tris[0],tris[1],tris[2], normals[0], normals[1], normals[2], true, tex));
+            triangles.push_back(new triangle(tris[0],tris[1],tris[2], normals[0], normals[1], normals[2], true, 
+                                             tex, alpha));
           } else {
             if(material_num == -1) {
               tex = new lambertian(new constant_texture(diffuse_materials[material_num]));
             } else {
               if(has_transparency[material_num]) {
-                tex = new dielectric(specular_materials[material_num], ior_materials[material_num], rng);;
+                tex = new dielectric(specular_materials[material_num], ior_materials[material_num], vec3(0,0,0),
+                                     0,  rng);;
               } else if(has_diffuse[material_num]) {
                 if(has_single_diffuse[material_num]) {
                   tex = new lambertian(new constant_texture(diffuse_materials[material_num]));
@@ -174,7 +208,7 @@ public:
                 tex = new lambertian(new constant_texture(vec3(1,1,1)));
               }
             }
-            triangles.push_back(new triangle(tris[0],tris[1],tris[2], true, tex));
+            triangles.push_back(new triangle(tris[0],tris[1],tris[2], true, tex, alpha));
           }
         }
       }
@@ -196,6 +230,7 @@ public:
       for (size_t s = 0; s < shapes.size(); s++) {
         n += shapes[s].mesh.num_face_vertices.size();
       }
+      alpha_texture *alpha = nullptr;
       obj_materials.reserve(materials.size()+1);
       // std::vector<Float* > obj_materials(materials.size()+1);
       std::vector<vec3 > diffuse_materials(materials.size()+1);
@@ -204,6 +239,7 @@ public:
       std::vector<bool > has_diffuse(materials.size()+1);
       std::vector<bool > has_transparency(materials.size()+1);
       std::vector<bool > has_single_diffuse(materials.size()+1);
+      std::vector<bool > has_alpha(materials.size()+1);
       std::vector<vec3 > image_dim(materials.size()+1);
       std::vector<int > nx_mat(materials.size()+1);
       std::vector<int > ny_mat(materials.size()+1);
@@ -219,18 +255,38 @@ public:
           nx_mat[i] = nx;
           ny_mat[i] = ny;
           nn_mat[i] = nn;
+          has_alpha[i] = false;
+          if(nn == 4) {
+            for(int j = 0; j < nx - 1; j++) {
+              for(int k = 0; k < ny - 1; k++) {
+                if(obj_materials[i][4*j + 4*nx*k + 3] != 1.0) {
+                  has_alpha[i] = true;
+                  break;
+                }
+              }
+              if(has_alpha[i]) {
+                break;
+              }
+            }
+          } 
         } else if (sizeof(materials[i].diffuse) == 12) {
           obj_materials.push_back(nullptr);
+          alpha_materials.push_back(nullptr);
+          
           diffuse_materials[i] = vec3(materials[i].diffuse[0],materials[i].diffuse[1],materials[i].diffuse[2]);
           has_diffuse[i] = true;
           has_single_diffuse[i] = true;
         } else {
           obj_materials.push_back(nullptr);
+          alpha_materials.push_back(nullptr);
+          
           has_diffuse[i] = false;
           has_single_diffuse[i] = false;
         }
         if(materials[i].dissolve < 1) {
           obj_materials.push_back(nullptr);
+          alpha_materials.push_back(nullptr);
+          
           specular_materials[i] = vec3(materials[i].diffuse[0],materials[i].diffuse[1],materials[i].diffuse[2]);
           ior_materials[i] = materials[i].ior;
           has_transparency[i] = true; 
@@ -239,7 +295,6 @@ public:
       bool has_normals = attrib.normals.size() > 0 ? true : false;
       vec3 tris[3];
       vec3 normals[3];
-      // vec3 colors[3];
       Float tx[3];
       Float ty[3];
       triangles.reserve(n+1);
@@ -264,10 +319,10 @@ public:
             if(idx.texcoord_index != -1) {
               tx[v] = attrib.texcoords[2*idx.texcoord_index+0];
               ty[v] = attrib.texcoords[2*idx.texcoord_index+1];
-              if(tx[v] < 0) tx[v] += 1;
-              if(ty[v] < 0) ty[v] += 1;
-              if(tx[v] > 1) tx[v] -= 1;
-              if(ty[v] > 1) ty[v] -= 1;
+              while(tx[v] < 0) tx[v] += 1;
+              while(ty[v] < 0) ty[v] += 1;
+              while(tx[v] > 1) tx[v] -= 1;
+              while(ty[v] > 1) ty[v] -= 1;
             }
           }
           
@@ -281,6 +336,17 @@ public:
           }
           // per-face material
           int material_num = shapes[s].mesh.material_ids[f];
+          if(material_num > -1) {
+            if(has_alpha[material_num]) {
+              alpha = new alpha_texture(obj_materials[material_num], nx, ny, nn, 
+                                        vec3(tx[0], tx[1], tx[2]), 
+                                        vec3(ty[0], ty[1], ty[2]));
+              alpha_materials.push_back(alpha);
+            } else {
+              alpha = nullptr;
+              alpha_materials.push_back(nullptr);
+            }
+          }
           
           material *tex;
           
@@ -289,7 +355,8 @@ public:
               tex = new orennayar(new constant_texture(vec3(1,1,1)), sigma);
             } else {
               if(has_transparency[material_num]) {
-                tex = new dielectric(specular_materials[material_num], ior_materials[material_num], rng);;
+                tex = new dielectric(specular_materials[material_num], ior_materials[material_num], vec3(0,0,0), 
+                                     0,  rng);;
               } else if(has_diffuse[material_num]) {
                 if(has_single_diffuse[material_num]) {
                   tex = new orennayar(new constant_texture(diffuse_materials[material_num]), sigma);
@@ -302,13 +369,15 @@ public:
                 tex = new orennayar(new constant_texture(vec3(1,1,1)), sigma);
               }
             }
-            triangles.push_back(new triangle(tris[0],tris[1],tris[2], normals[0], normals[1], normals[2], true, tex));
+            triangles.push_back(new triangle(tris[0],tris[1],tris[2], normals[0], normals[1], normals[2], true, 
+                                             tex,alpha));
           } else {
             if(material_num == -1) {
               tex = new orennayar(new constant_texture(diffuse_materials[material_num]), sigma);
             } else {
               if(has_transparency[material_num]) {
-                tex = new dielectric(specular_materials[material_num], ior_materials[material_num], rng);;
+                tex = new dielectric(specular_materials[material_num], ior_materials[material_num], vec3(0,0,0), 
+                                     0,  rng);;
               } else if(has_diffuse[material_num]) {
                 if(has_single_diffuse[material_num]) {
                   tex = new orennayar(new constant_texture(diffuse_materials[material_num]), sigma);
@@ -321,7 +390,7 @@ public:
                 tex = new orennayar(new constant_texture(vec3(1,1,1)), sigma);
               }
             }
-            triangles.push_back(new triangle(tris[0],tris[1],tris[2], true, tex));
+            triangles.push_back(new triangle(tris[0],tris[1],tris[2], true, tex, alpha));
           }
         }
       }
@@ -346,8 +415,6 @@ public:
       bool has_normals = attrib.normals.size() > 0 ? true : false;
       vec3 tris[3];
       vec3 normals[3];
-      // vec3 colors[3];
-      // std::vector<hitable* > triangles;
       triangles.reserve(n+1);
       for (size_t s = 0; s < shapes.size(); s++) {
         size_t index_offset = 0;
@@ -374,13 +441,100 @@ public:
             n--;
             continue;
           }
+          if((normals[0].x() == 0 && normals[0].y() == 0 && normals[0].z() == 0) ||
+             (normals[1].x() == 0 && normals[1].y() == 0 && normals[1].z() == 0) ||
+             (normals[2].x() == 0 && normals[2].y() == 0 && normals[2].z() == 0)) {
+            has_normals = false;
+          }
+          
           if(has_normals) {
             triangles.push_back(new triangle(tris[0],tris[1],tris[2],
                                              normals[0],normals[1],normals[2], 
                                              false,
-                                             mat_ptr));
+                                             mat_ptr, nullptr));
           } else {
-            triangles.push_back(new triangle(tris[0],tris[1],tris[2], false, mat_ptr));
+            triangles.push_back(new triangle(tris[0],tris[1],tris[2], false, mat_ptr, nullptr));
+          }
+        }
+      }
+      tri_mesh_bvh = new bvh_node(&triangles[0], n, shutteropen, shutterclose, rng);
+    }
+  };
+  trimesh(std::string inputfile, std::string basedir, float vertex_color_sigma,
+          Float scale, bool is_vertex_color, Float shutteropen, Float shutterclose, random_gen rng) {
+    tinyobj::attrib_t attrib;
+    std::vector<tinyobj::shape_t > shapes;
+    std::vector<tinyobj::material_t > materials;
+    std::string warn, err;
+    mat_ptr = nullptr;
+    
+    bool is_lamb = vertex_color_sigma == 0 ? true : false;
+    
+    bool ret = tinyobj::LoadObj(&attrib, &shapes, &materials, &warn, &err, inputfile.c_str(), basedir.c_str());
+    //need to define else
+    if(ret) {
+      int n = 0;
+      for (size_t s = 0; s < shapes.size(); s++) {
+        n += shapes[s].mesh.num_face_vertices.size();
+      }
+      bool has_normals = attrib.normals.size() > 0 ? true : false;
+      bool has_vertex_colors = attrib.colors.size() > 0 ? true : false;
+      vec3 tris[3];
+      vec3 normals[3];
+      vec3 colors[3];
+      triangles.reserve(n+1);
+      for (size_t s = 0; s < shapes.size(); s++) {
+        size_t index_offset = 0;
+        for (size_t f = 0; f < shapes[s].mesh.num_face_vertices.size(); f++) {
+          colors[0] = vec3(1,1,1);
+          colors[1] = vec3(1,1,1);
+          colors[2] = vec3(1,1,1);
+          for (size_t v = 0; v < 3; v++) {
+            tinyobj::index_t idx = shapes[s].mesh.indices[index_offset + v];
+            
+            tris[v] = vec3(attrib.vertices[3*idx.vertex_index+0],
+                           attrib.vertices[3*idx.vertex_index+1],
+                           attrib.vertices[3*idx.vertex_index+2])*scale;
+            if(has_vertex_colors) {
+              colors[v] = vec3(attrib.colors[3*idx.vertex_index+0],
+                               attrib.colors[3*idx.vertex_index+1],
+                               attrib.colors[3*idx.vertex_index+2]);
+            }
+            if(has_normals) {
+              normals[v] = vec3(attrib.normals[3*idx.normal_index+0],
+                                attrib.normals[3*idx.normal_index+1],
+                                attrib.normals[3*idx.normal_index+2]);
+            }
+          }
+          
+          index_offset += 3;
+          if(std::isnan(tris[0].x()) || std::isnan(tris[0].y()) || std::isnan(tris[0].z()) ||
+             std::isnan(tris[1].x()) || std::isnan(tris[1].y()) || std::isnan(tris[1].z()) ||
+             std::isnan(tris[2].x()) || std::isnan(tris[2].y()) || std::isnan(tris[2].z())) {
+            // triangles.pop_back();
+            n--;
+            continue;
+          }
+          if((normals[0].x() == 0 && normals[0].y() == 0 && normals[0].z() == 0) ||
+             (normals[1].x() == 0 && normals[1].y() == 0 && normals[1].z() == 0) ||
+             (normals[2].x() == 0 && normals[2].y() == 0 && normals[2].z() == 0)) {
+            has_normals = false;
+          }
+          material* tex;
+          if(is_lamb) {
+            tex = new lambertian(new triangle_texture(colors[0],colors[1],colors[2]));
+          } else {
+            tex = new orennayar(new triangle_texture(colors[0],colors[1],colors[2]),
+                                vertex_color_sigma);
+          }
+          
+          if(has_normals) {
+            triangles.push_back(new triangle(tris[0],tris[1],tris[2],
+                                             normals[0],normals[1],normals[2], 
+                                             true,
+                                             tex, nullptr));
+          } else {
+            triangles.push_back(new triangle(tris[0],tris[1],tris[2], true, tex, nullptr));
           }
         }
       }
@@ -396,6 +550,7 @@ public:
   bvh_node* tri_mesh_bvh;
   material *mat_ptr;
   std::vector<Float* > obj_materials;
+  std::vector<alpha_texture* > alpha_materials;
   std::vector<hitable* > triangles;
 };
 
