@@ -2,15 +2,21 @@
 #define RECTH
 
 #include "hitable.h"
+#include "material.h"
 
 class xy_rect : public hitable {
 public:
   xy_rect() {}
-  xy_rect(Float _x0, Float _x1, Float _y0, Float _y1, Float _k, material *mat) :
-    x0(_x0), x1(_x1), y0(_y0), y1(_y1), k(_k), mp(mat) {};
+  xy_rect(Float _x0, Float _x1, Float _y0, Float _y1, Float _k, 
+          material *mat, alpha_texture* alpha_mask, bool flipped) :
+    x0(_x0), x1(_x1), y0(_y0), y1(_y1), k(_k), mp(mat), alpha_mask(alpha_mask), flipped(flipped) {};
+  ~xy_rect() {
+    delete alpha_mask;
+    delete mp;
+  }
   virtual bool hit(const ray& r, Float t_min, Float t_max, hit_record& rec, random_gen& rng);
   virtual bool bounding_box(Float t0, Float t1, aabb& box) const {
-    box = aabb(vec3(x0,y0,k-0.0001), vec3(x1,y1,k+0.0001));
+    box = aabb(vec3(x0,y0,k-0.001), vec3(x1,y1,k+0.001));
     return(true);
   }
   virtual Float pdf_value(const vec3& o, const vec3& v, random_gen& rng) {
@@ -30,16 +36,23 @@ public:
   }
   Float x0, x1, y0, y1, k;
   material *mp;
+  alpha_texture *alpha_mask;
+  bool flipped;
 };
 
 class xz_rect : public hitable {
 public:
   xz_rect() {}
-  xz_rect(Float _x0, Float _x1, Float _z0, Float _z1, Float _k, material *mat) :
-  x0(_x0), x1(_x1), z0(_z0), z1(_z1), k(_k), mp(mat) {};
+  xz_rect(Float _x0, Float _x1, Float _z0, Float _z1, Float _k, 
+          material *mat, alpha_texture* alpha_mask, bool flipped) :
+  x0(_x0), x1(_x1), z0(_z0), z1(_z1), k(_k), mp(mat), alpha_mask(alpha_mask), flipped(flipped) {};
+  ~xz_rect() {
+    delete alpha_mask;
+    delete mp;
+  }
   virtual bool hit(const ray& r, Float t_min, Float t_max, hit_record& rec, random_gen& rng);
   virtual bool bounding_box(Float t0, Float t1, aabb& box) const {
-    box = aabb(vec3(x0,k-0.0001,z0), vec3(x1,k+0.0001,z1));
+    box = aabb(vec3(x0,k-0.001,z0), vec3(x1,k+0.001,z1));
     return(true);
   }
   virtual Float pdf_value(const vec3& o, const vec3& v, random_gen& rng) {
@@ -59,16 +72,23 @@ public:
   }
   Float x0, x1, z0, z1, k;
   material *mp;
+  alpha_texture *alpha_mask;
+  bool flipped;
 };
 
 class yz_rect : public hitable {
 public:
   yz_rect() {}
-  yz_rect(Float _y0, Float _y1, Float _z0, Float _z1, Float _k, material *mat) :
-  y0(_y0), y1(_y1), z0(_z0), z1(_z1), k(_k), mp(mat) {};
+  yz_rect(Float _y0, Float _y1, Float _z0, Float _z1, Float _k, 
+          material *mat, alpha_texture* alpha_mask, bool flipped) :
+  y0(_y0), y1(_y1), z0(_z0), z1(_z1), k(_k), mp(mat), alpha_mask(alpha_mask), flipped(flipped) {};
+  ~yz_rect() {
+    delete alpha_mask;
+    delete mp;
+  }
   virtual bool hit(const ray& r, Float t_min, Float t_max, hit_record& rec, random_gen& rng);
   virtual bool bounding_box(Float t0, Float t1, aabb& box) const {
-    box = aabb(vec3(k-0.0001,y0,z0), vec3(k+0.0001,y1,z1));
+    box = aabb(vec3(k-0.001,y0,z0), vec3(k+0.001,y1,z1));
     return(true);
   }
   virtual Float pdf_value(const vec3& o, const vec3& v, random_gen& rng) {
@@ -88,6 +108,8 @@ public:
   }
   Float y0, y1, z0, z1, k;
   material *mp;
+  alpha_texture *alpha_mask;
+  bool flipped;
 };
 
 bool xy_rect::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, random_gen& rng) {
@@ -100,14 +122,26 @@ bool xy_rect::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, rando
   if(x < x0 || x > x1 || y < y0 || y > y1) {
     return(false);
   }
-  rec.u = (x-x0)/(x1-x0);
-  rec.v = (y-y0)/(y1-y0);
+  Float u = (x-x0)/(x1-x0);
+  Float v = (y-y0)/(y1-y0);
+  if(flipped) {
+    u = 1 - u;
+  }
+  if(alpha_mask) {
+    if(alpha_mask->value(u, v, rec.p).x() < rng.unif_rand()) {
+      return(false);
+    }
+    rec.normal = dot(r.direction(),vec3(0,0,1)) < 0 ? vec3(0,0,1) : vec3(0,0,-1);
+  } else {
+    rec.normal = flipped ? vec3(0,0,-1) : vec3(0,0,1);
+  }
+  rec.u = u;
+  rec.v = v;
   rec.t = t;
   rec.mat_ptr = mp;
   rec.p = r.point_at_parameter(t);
   rec.p.e[2] = k;
-  
-  rec.normal = vec3(0,0,1);
+
   return(true);
 }
 
@@ -121,13 +155,26 @@ bool xz_rect::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, rando
   if(x < x0 || x > x1 || z < z0 || z > z1) {
     return(false);
   }
-  rec.u = (x-x0)/(x1-x0);
-  rec.v = (z-z0)/(z1-z0);
+  Float u = 1-(x-x0)/(x1-x0);
+  Float v = (z-z0)/(z1-z0);
+  if(flipped) {
+    u = 1 - u;
+  }
+  if(alpha_mask) {
+    if(alpha_mask->value(u, v, rec.p).x() < rng.unif_rand()) {
+      return(false);
+    }
+    rec.normal =  dot(r.direction(),vec3(0,1,0)) < 0 ? vec3(0,1,0) : vec3(0,-1,0);
+  } else {
+    rec.normal =  flipped ? vec3(0,-1,0) : vec3(0,1,0);
+  }
+  rec.u = u;
+  rec.v = v;
   rec.t = t;
   rec.mat_ptr = mp;
   rec.p = r.point_at_parameter(t);
   rec.p.e[1] = k;
-  rec.normal = vec3(0,1,0);
+
   return(true);
 }
 
@@ -141,13 +188,26 @@ bool yz_rect::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, rando
   if(z < z0 || z > z1 || y < y0 || y > y1) {
     return(false);
   }
-  rec.u = (z-z0)/(z1-z0);
-  rec.v = (y-y0)/(y1-y0);
+  Float u = 1-(z-z0)/(z1-z0);
+  Float v = (y-y0)/(y1-y0);
+  if(flipped) {
+    u = 1 - u;
+  }
+  if(alpha_mask) {
+    if(alpha_mask->value(u, v, rec.p).x() < rng.unif_rand()) {
+      return(false);
+    }
+    rec.normal =  dot(r.direction(),vec3(1,0,0)) < 0 ? vec3(1,0,0) : vec3(-1,0,0);
+  } else {
+    rec.normal =  flipped ? vec3(-1,0,0) : vec3(1,0,0);
+  }
+  rec.u = u;
+  rec.v = v;
   rec.t = t;
   rec.mat_ptr = mp;
   rec.p = r.point_at_parameter(t);
   rec.p.e[0] = k;
-  rec.normal = vec3(1,0,0);
+
   return(true);
 }
 
